// premiere implementation
function x0 = dichoto_suite(f, a, b, nbetap)
fa = f(a);
fb = f(b);
if fa*fb>0,error('erreur : f a le meme signe en a et en b');end   
x0=zeros(nbetap,1);    
for k=1:nbetap
    c=(a+b)/2;
    fc = f(c);
    if fc*fa<1e-15 
        b=c;
    else
        fa=fc;
        a=c;
    end    
    x0(k)=c;   
    //printf('iteration k = %d: a= %f, b= %f\n', k,a,b)  
end   
end

function xs = secante_suite(f, x0, x1, nbetap)  
xs=zeros(nbetap,1); 
xs(1)=x0;
xs(2)=x1;
for k=2:nbetap-1
    //xs(k+1)= xs(k) - (xs(k) - xs(k-1))/(f(xs(k)) - f(xs(k-1)))*f(xs(k));//implementation standard
    xs(k+1)= xs(k) - (xs(k) - xs(k-1))/safe_division(f(xs(k)) - f(xs(k-1)))*f(xs(k)); //implementation amelioree...
    disp(safe_division(f(xs(k)) - f(xs(k-1))))
    //printf('iteration k = %d: xs= %f, b= %f\n', xs(k+1),f(xs(k)),safe_division(f(xs(k)) - f(xs(k-1))))  
end        
end


function xs = newton_suite(f, df, x0, nbetap)
xs=zeros(nbetap,1); 
xs(1)=x0;
for k=1:nbetap-1
    //xs(k+1)= xs(k) - f(xs(k))/( df(xs(k)) );//implementation standard
    xs(k+1)= xs(k) - f(xs(k))/safe_division( df(xs(k)) );//implementation amelioree...
end           
end


function xs = newton_corr_suite(f, df, x0, nbetap,m)
xs=zeros(nbetap,1); 
xs(1)=x0;
for k=1:nbetap-1
    //xs(k+1)= xs(k) - m*f(xs(k))/df(xs(k));
    xs(k+1)= xs(k) - m*f(xs(k))/safe_division( df(xs(k)) );
end           
end



// deuxieme implementation
function [c,nbr_its] = dichoto(f, a, b, maxit,tol) 
fa = f(a);fb = f(b);
if fa*fb>0, error('erreur : f a le meme signe en a et en b');end 
c=(a+b)/2;
fc = f(c); 
for k=1:maxit-1  
    if fc*fa<0, b=c;
    else,       a=c;fa=fc;
    end  
    c=(a+b)/2;  
    fc = f(c);   
    if abs(fc)<tol
        break ;
    end 
end  
nbr_its=k+1; //convention: regarder dichoto_suite
endfunction


function [zeta,nbr_its] = secante(f, x0, x1, maxit,tol) 
zeta=x1;
zetaold=x0;
for k=3:maxit-1  
    //zeta_new= zeta - (zeta - zetaold)/(f(zeta) - f(zetaold))*f(zeta);
    zeta_new= zeta - (zeta - zetaold)/safe_division(f(zeta) - f(zetaold))*f(zeta);
    zetaold=zeta;
    zeta=zeta_new;    
    if abs(f(zeta))<tol
        break ;
    end 
end  
nbr_its=k; //convention: regarder secante_suite
endfunction


function [zeta,nbr_its] = newton(f, df, x0, maxit,tol) 
zeta=x0;
D = length(x0);

for k=2:maxit-1  
    if D==1
        zeta= zeta - f(zeta)/safe_division( df(zeta) );
    else
        zeta= zeta - df(zeta)\f(zeta);//pour des problemes 1D, ca marche aussi... 
    end    
    if norm(abs(f(zeta)))<tol
        break ;
    end 
end  
nbr_its=k; //convention: regarder newton_suite
endfunction

function [zeta,nbr_its] = newton_corr(f, df, x0, maxit,tol,m) 
//methode de Newton avec correction
zeta=x0;
for k=2:maxit-1  
    zeta= zeta - m*f(zeta)/safe_division( df(zeta) );//pour des problemes 1D, ca marche aussi... 
    //zeta= zeta - df(zeta)\f(zeta);
    if abs(f(zeta))<tol
        break ;
    end 
end  
nbr_its=k; //convention: regarder newton_suite
endfunction


function [er] = validate_derivative(myfx,mydfx,x)
D=size(x,1);    
dx = rand(D,1);    
epsilon=1e-8;
fx = myfx(x);
fxeps = myfx(x+epsilon*dx);
dfx = mydfx(x);
e = 1/epsilon*(fxeps-fx) -     dfx*dx;
er=norm(e)/norm(dfx*dx);
endfunction

function [y] = safe_division(x)
//cette fonction est utilisée pour éviter la division par zéro
if abs(x)<1e-14 then
    y=1e-14;
else
    y=x;
end
//y = sign(x+1e-15)*max(abs(x),1e-10);    
endfunction








